@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: HTML-TEMPLATE; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/html-template/api.lisp,v 1.18 2006/01/03 18:41:44 edi Exp $
+;;; $Header: /usr/local/cvsrep/html-template/api.lisp,v 1.20 2006/09/29 22:36:26 edi Exp $
 
 ;;; Copyright (c) 2003-2006, Dr. Edmund Weitz. All rights reserved.
 
@@ -57,7 +57,8 @@ Keyword arguments will only be accepted if TEMPLATE is a PATHNAME."))
 
 (defmethod create-template-printer ((pathname pathname)
                                     &key (force *force-default*)
-                                         (element-type 'character)
+                                         (element-type #-:lispworks 'character
+                                                       #+:lispworks 'lw:simple-char)
                                          (if-does-not-exist :error)
                                          (external-format :default))
   (let* ((merged-pathname (merge-pathnames pathname
@@ -119,7 +120,7 @@ used if TEMPLATE/PRINTER is a pathname."))
     (funcall function values)))
 
 (defmethod fill-and-print-template ((string string) values
-                                     &rest rest
+                                    &rest rest
                                     &key (stream *default-template-output*))
   (remf rest :stream)
   (when rest
