@@ -1,5 +1,5 @@
 ;;; -*- Mode: LISP; Syntax: COMMON-LISP; Package: HTML-TEMPLATE; Base: 10 -*-
-;;; $Header: /usr/local/cvsrep/html-template/template.lisp,v 1.19 2006/04/03 08:35:53 edi Exp $
+;;; $Header: /usr/local/cvsrep/html-template/template.lisp,v 1.22 2006/09/29 22:36:26 edi Exp $
 
 ;;; Copyright (c) 2003-2006, Dr. Edmund Weitz. All rights reserved.
 
@@ -45,7 +45,10 @@ associated with SYMBOL isn't to our liking."
                              ,symbol)
                      (multiple-value-list (eval (read))))
       other-value)))
-    
+
+;; indentation for LispWorks editor
+#+:lispworks
+(editor:setup-indent "with-use-value-restart" 1 2 4)
 
 (defun create-simple-printer (string-list &optional symbol/pathname (next-fn #'no-values))
   "Used internally to create template printers for TMPL_VAR,
@@ -65,25 +68,24 @@ printed first."
         ;; TMPL_VAR tag
         (lambda (values)
           (write-string string *template-output*)
-          (write-string (let ((value (funcall *value-access-function*
-                                              symbol/pathname values)))
-                          (typecase value
-                            (null
-                              (if *convert-nil-to-empty-string*
-                                ""
-                                (with-use-value-restart (symbol/pathname)
-                                  (signal-template-missing-value-error
-                                   "Value for symbol ~S is NIL"
-                                   symbol/pathname))))
-                            (string
-                              value)
-                            (otherwise
+          (let* ((value (funcall *value-access-function* symbol/pathname values))
+                 (string (typecase value
+                           (null
+                            (if *convert-nil-to-empty-string*
+                              ""
                               (with-use-value-restart (symbol/pathname)
-                                (error 'template-not-a-string-error
-                                       :value value
-                                       :format-control "Value ~S for symbol ~S is not a string"
-                                       :format-arguments (list value symbol/pathname))))))
-                        *template-output*)
+                                (signal-template-missing-value-error 
+                                 "Value for symbol ~S is NIL"
+                                 symbol/pathname))))
+                           (string value)                            
+                           (otherwise
+                            (cond (*format-non-strings* (format nil "~A" value))
+                                  (t (with-use-value-restart (symbol/pathname)
+                                       (error 'template-not-a-string-error
+                                              :value value
+                                              :format-control "Value ~S for symbol ~S is not a string"
+                                              :format-arguments (list value symbol/pathname)))))))))
+            (write-string (funcall *string-modifier* string) *template-output*))
           (funcall next-fn values)))
       (pathname
         ;; TMPL_INCLUDE tag
@@ -109,9 +111,9 @@ printed first.  If UNLESSP is true, IF-FN and ELSE-FN are switched."
         (funcall else-fn values))
       (funcall next-fn values))))
 
-(defun create-loop-printer (string-list symbol loop-fn next-fn)
+(defun create-loop-printer (string-list symbol body-fn next-fn)
   "Used internally to create template printers for TMPL_LOOP
-tags. SYMBOL is the symbol associated with the tag. LOOP-FN is the
+tags. SYMBOL is the symbol associated with the tag. BODY-FN is the
 template printer for the body of the loop. NEXT-FN is the next
 function to be called in the chain of closures. STRING-LIST is a list
 of strings in reverse order to be printed first."
@@ -120,27 +122,42 @@ of strings in reverse order to be printed first."
             (lambda (values)
               (write-string string *template-output*)
               (dolist (value (funcall *value-access-function*
-                                      symbol values))
-                (funcall loop-fn value))
+                                      symbol values t))
+                (funcall body-fn value))
               (funcall next-fn values)))
           (t
             (lambda (values)
               (write-string string *template-output*)
               (loop for value across (funcall *value-access-function*
-                                              symbol values)
-                    do (funcall loop-fn value))
+                                              symbol values t)
+                    do (funcall body-fn value))
               (funcall next-fn values))))))
 
+(defun create-repeat-printer (string-list symbol body-fn next-fn)
+  "Used internally to create template printers for TMPL_REPEAT
+tags. SYMBOL is the symbol associated with the tag. BODY-FN is the
+template printer for the body of the loop. NEXT-FN is the next
+function to be called in the chain of closures. STRING-LIST is a list
+of strings in reverse order to be printed first."
+  (let ((string (list-to-string string-list)))
+    (lambda (values)
+      (write-string string *template-output*)
+      (let ((factor (funcall *value-access-function* symbol values)))
+        (when (and (integerp factor) (plusp factor))
+          (loop repeat factor
+                do (funcall body-fn values))))
+      (funcall next-fn values))))
+
 (defun create-template-printer-aux (string-stack end-token)
   "Reads from *STANDARD-INPUT* and returns a template printer from
 what it reads.  When this function is entered the stream pointer must
 not be inside a template tag.  STRING-STACK is a list of strings (in
 reverse order) read so far which haven't been used to build a template
-printer.  END-TOKEN is either NIL or one of :LOOP, :IF, :IF-ELSE, or
-:UNLESS-ELSE denoting that we expect certain tags to close open
-TMPL_LOOP, TMPL_IF, or TMPL_UNLESS tags.  This function returns a
-second value which is true if, after reading TMPL_IF or TMPL_UNLESS, a
-corresponding TMPL_ELSE was seen."
+printer.  END-TOKEN is either NIL or one of :LOOP, :REPEAT, :IF,
+:IF-ELSE, or :UNLESS-ELSE denoting that we expect certain tags to
+close open TMPL_LOOP, TMPL_REPEAT, TMPL_IF, or TMPL_UNLESS tags.  This
+function returns a second value which is true if, after reading
+TMPL_IF or TMPL_UNLESS, a corresponding TMPL_ELSE was seen."
   (let* ((string
            ;; read text up to the next template start marker
            (read-until *template-start-marker*
@@ -155,6 +172,7 @@ corresponding TMPL_ELSE was seen."
                                         "Unexpected EOF, ~A tag is missing"
                                         (case end-token
                                           ((:loop) "<!-- /TMPL_LOOP -->")
+                                          ((:repeat) "<!-- /TMPL_REPEAT -->")
                                           ((:if :if-else) "<!-- /TMPL_IF -->")
                                           ((:unless :unless-else) "<!-- /TMPL_UNLESS -->"))))
                                      ;; otherwise (EOF before another
@@ -231,20 +249,22 @@ corresponding TMPL_ELSE was seen."
                                         symbol
                                         next-fn)
                  else-follows))))
-          ((string-equal token "TMPL_LOOP")
-            ;; TMPL_LOOP tag - first read the symbol which has to
-            ;; follow and intern it
-            (let* ((symbol (read-tag-rest :read-attribute t))
+          ((or (string-equal token "TMPL_LOOP")
+               (string-equal token "TMPL_REPEAT"))
+            ;; TMPL_LOOP or TMPL_REPEAT tag - first read the symbol
+            ;; which has to follow and intern it
+            (let* ((kind (if (string-equal token "TMPL_LOOP") :loop :repeat))
+                   (symbol (read-tag-rest :read-attribute t))
                    ;; then read the stream up to the corresponding
-                   ;; /TMPL_LOOP and create a template printer for the
+                   ;; end tag and create a template printer for the
                    ;; loop body
-                   (loop-fn (with-syntax-error-location ()
+                   (body-fn (with-syntax-error-location ()
                               (create-template-printer-aux
                                (skip-trailing-whitespace)
                                ;; this argument denotes that we expect
-                               ;; to see /TMPL_LOOP and want to stop
-                               ;; there
-                               :loop))))
+                               ;; to see /TMPL_LOOP or /TMPL_REPEAT and
+                               ;; want to stop there
+                               kind))))
               (multiple-value-bind (next-fn else-follows)
                   ;; now we recursively create the template printer
                   ;; for the rest of the stream
@@ -254,11 +274,14 @@ corresponding TMPL_ELSE was seen."
                  ;; then we combine it with the strings before the tag
                  ;; and the body printer to create a template printer
                  ;; for TMPL_LOOP
-                 (create-loop-printer (cons (skip-leading-whitespace string)
-                                            string-stack)
-                                      symbol
-                                      loop-fn
-                                      next-fn)
+                 (funcall (case kind
+                            (:loop #'create-loop-printer)
+                            (:repeat #'create-repeat-printer))
+                          (cons (skip-leading-whitespace string)
+                                string-stack)
+                          symbol
+                          body-fn
+                          next-fn)
                  else-follows))))
           ((string-equal token "/TMPL_LOOP")
             (unless (eq end-token :loop)
@@ -273,6 +296,19 @@ corresponding TMPL_ELSE was seen."
             ;; this is the end of some TMPL_LOOP body
             (create-simple-printer (cons (skip-leading-whitespace string)
                                          string-stack)))
+          ((string-equal token "/TMPL_REPEAT")
+            (unless (eq end-token :repeat)
+              ;; check if we expected /TMPL_REPEAT here, i.e. if an open
+              ;; TMPL_REPEAT was pending
+              (with-syntax-error-location ()
+                (signal-template-syntax-error "Unexpected /TMPL_REPEAT")))
+            ;; read the rest of the tag but ignore it - no attributes
+            ;; expected
+            (read-tag-rest)
+            ;; just create a simple template printer for strings -
+            ;; this is the end of some TMPL_REPEAT body
+            (create-simple-printer (cons (skip-leading-whitespace string)
+                                         string-stack)))
           ((or (string-equal token "TMPL_IF")
                (string-equal token "TMPL_UNLESS"))
             ;; TMPL_IF or TMPL_UNLESS tag - first read the symbol
